@@ -72,6 +72,7 @@
 #include "mlmmj_tests.h"
 #include "send_help.h"
 #include "gethdrline.h"
+#include "unistr.h"
 
 ATF_TC_WITHOUT_HEAD(random_int);
 ATF_TC_WITHOUT_HEAD(chomp);
@@ -158,6 +159,10 @@ ATF_TC_WITHOUT_HEAD(gethdrline);
 ATF_TC_WITHOUT_HEAD(readlf);
 ATF_TC_WITHOUT_HEAD(mod_get_addr_type);
 ATF_TC_WITHOUT_HEAD(send_probe);
+ATF_TC_WITHOUT_HEAD(parse_content_type);
+ATF_TC_WITHOUT_HEAD(find_in_list);
+ATF_TC_WITHOUT_HEAD(do_access);
+ATF_TC_WITHOUT_HEAD(unistr_header_to_utf8);
 
 ATF_TC_BODY(random_int, tc)
 {
@@ -678,16 +683,17 @@ ATF_TC_BODY(strtotimet, tc)
 
 ATF_TC_BODY(decode_qp, tc)
 {
-	ATF_REQUIRE_STREQ(decode_qp("="), "=");
-	ATF_REQUIRE_STREQ(decode_qp("=\ra"), "=\ra");
-	ATF_REQUIRE_STREQ(decode_qp("=\r\na"), "a");
-	ATF_REQUIRE_STREQ(decode_qp("=\na"), "a");
-	ATF_REQUIRE_STREQ(decode_qp("This is a subject"), "This is a subject");
-	ATF_REQUIRE_STREQ(decode_qp("This= is a subject"), "This= is a subject");
-	ATF_REQUIRE_STREQ(decode_qp("This=2 is a subject"), "This=2 is a subject");
-	ATF_REQUIRE_STREQ(decode_qp("This=23 is a subject"), "This# is a subject");
-	ATF_REQUIRE_STREQ(decode_qp("This=3D is a subject"), "This= is a subject");
-	ATF_REQUIRE_STREQ(decode_qp("This_ is a subject"), "This_ is a subject");
+	ATF_REQUIRE_STREQ(decode_qp("=", false), "=");
+	ATF_REQUIRE_STREQ(decode_qp("=\ra", false), "=\ra");
+	ATF_REQUIRE_STREQ(decode_qp("=\r\na", false), "a");
+	ATF_REQUIRE_STREQ(decode_qp("=\na", false), "a");
+	ATF_REQUIRE_STREQ(decode_qp("This is a subject", false), "This is a subject");
+	ATF_REQUIRE_STREQ(decode_qp("This= is a subject", false), "This= is a subject");
+	ATF_REQUIRE_STREQ(decode_qp("This=2 is a subject", false), "This=2 is a subject");
+	ATF_REQUIRE_STREQ(decode_qp("This=23 is a subject", false), "This# is a subject");
+	ATF_REQUIRE_STREQ(decode_qp("This=3D is a subject", false), "This= is a subject");
+	ATF_REQUIRE_STREQ(decode_qp("This_ is a subject", false), "This_ is a subject");
+	ATF_REQUIRE_STREQ(decode_qp("This_ is a subject", true), "This  is a subject");
 }
 
 ATF_TC_BODY(parse_lastdigest, tc)
@@ -2861,6 +2867,144 @@ ATF_TC_BODY(send_probe, tc)
 	}
 }
 
+ATF_TC_BODY(find_in_list, tc)
+{
+	strlist bla = tll_init();
+	ATF_REQUIRE(find_in_list(NULL, NULL) == NULL);
+	ATF_REQUIRE(find_in_list(&bla, NULL) == NULL);
+	ATF_REQUIRE(find_in_list(&bla, "test") == NULL);
+	tll_push_back(bla, NULL);
+	ATF_REQUIRE(find_in_list(&bla, "test") == NULL);
+	tll_push_back(bla, "plop");
+	ATF_REQUIRE(find_in_list(&bla, "test") == NULL);
+	tll_push_back(bla, "test");
+	ATF_REQUIRE_STREQ(find_in_list(&bla, "test"), "test");
+	tll_pop_back(bla);
+	tll_push_back(bla, "test longer");
+	ATF_REQUIRE_STREQ(find_in_list(&bla, "test"), "test longer");
+}
+
+ATF_TC_BODY(parse_content_type, tc)
+{
+	char *m, *b;
+	strlist hdrs = tll_init();
+	parse_content_type(NULL, &m, &b);
+	ATF_REQUIRE(m == NULL);
+	ATF_REQUIRE(b == NULL);
+	tll_push_back(hdrs, "From: plop");
+	parse_content_type(&hdrs, &m, &b);
+	ATF_REQUIRE(m == NULL);
+	ATF_REQUIRE(b == NULL);
+	tll_push_back(hdrs, "Content-Type:");
+	parse_content_type(&hdrs, &m, &b);
+	ATF_REQUIRE(m == NULL);
+	ATF_REQUIRE(b == NULL);
+	tll_pop_back(hdrs);
+	tll_push_back(hdrs, "Content-Type: multipart/mixed");
+	parse_content_type(&hdrs, &m, &b);
+	ATF_REQUIRE_STREQ(m, "multipart/mixed");
+	free(m);
+	ATF_REQUIRE(b == NULL);
+	tll_pop_back(hdrs);
+	tll_push_back(hdrs, "Content-Type: multipart/mixed;");
+	parse_content_type(&hdrs, &m, &b);
+	ATF_REQUIRE_STREQ(m, "multipart/mixed");
+	free(m);
+	ATF_REQUIRE(b == NULL);
+	tll_pop_back(hdrs);
+	tll_push_back(hdrs, "Content-Type: multipart/mixed;\tboundary=bla");
+	parse_content_type(&hdrs, &m, &b);
+	ATF_REQUIRE_STREQ(m, "multipart/mixed");
+	free(m);
+	ATF_REQUIRE_STREQ(b,"bla");
+	free(b);
+	tll_pop_back(hdrs);
+	tll_push_back(hdrs, "Content-Type: multipart/mixed;\tboundary=bla;");
+	parse_content_type(&hdrs, &m, &b);
+	ATF_REQUIRE_STREQ(m, "multipart/mixed");
+	free(m);
+	ATF_REQUIRE_STREQ(b,"bla");
+	free(b);
+}
+
+ATF_TC_BODY(do_access, tc)
+{
+	strlist rules = tll_init();
+	strlist headers = tll_init();
+	tll_push_back(rules, "allow");
+	char *msg = NULL;
+	char *qualifier = NULL;
+
+	ATF_REQUIRE_EQ(do_access(NULL, NULL, "test@plop", &msg, &qualifier), ACT_ALLOW);
+	ATF_REQUIRE(msg == NULL);
+	ATF_REQUIRE(qualifier == NULL);
+
+	ATF_REQUIRE_EQ(do_access(&rules, NULL, "test@plop", &msg, &qualifier), ACT_ALLOW);
+	ATF_REQUIRE_STREQ(msg, "access - A mail from \"test@plop\" was allowed by rule #0 \"allow\"");
+	ATF_REQUIRE(qualifier == NULL);
+	tll_pop_back(rules);
+	tll_push_back(rules, "deny");
+	ATF_REQUIRE_EQ(do_access(&rules, NULL, "test@plop", &msg, &qualifier), ACT_DENY);
+	ATF_REQUIRE_STREQ(msg, "access - A mail from \"test@plop\" was denied by rule #0 \"deny\"");
+	ATF_REQUIRE(qualifier == NULL);
+	tll_pop_back(rules);
+	tll_push_back(rules, "crap");
+	ATF_REQUIRE_EQ(do_access(&rules, NULL, "test@plop", &msg, &qualifier), ACT_DENY);
+	ATF_REQUIRE_STREQ(msg, "Unable to parse rule #0 \"crap\": Missing action keyword. Denying post from \"test@plop\"");
+	ATF_REQUIRE(qualifier == NULL);
+	tll_pop_back(rules);
+	tll_push_back(headers, "from: toto@bla");
+	tll_push_back(rules, "moderate ^from: toto@");
+	ATF_REQUIRE_EQ(do_access(&rules, &headers, "test@plop", &msg, &qualifier), ACT_MODERATE);
+	ATF_REQUIRE_STREQ(msg, "access - A mail from \"test@plop\" with header \"from: toto@bla\" was moderated by rule #0 \"moderate ^from: toto@\"");
+	ATF_REQUIRE(qualifier == NULL);
+	tll_push_front(rules, "moderatemeh ^from: toto@");
+	ATF_REQUIRE_EQ(do_access(&rules, &headers, "test@plop", &msg, &qualifier), ACT_DENY);
+	ATF_REQUIRE_STREQ(msg, "Unable to parse rule #0 \"moderatemeh ^from: toto@\": Invalid character after action keyword. Denying post from \"test@plop\"");
+	ATF_REQUIRE(qualifier == NULL);
+	tll_push_front(rules, "moderate- ^from: toto@");
+	ATF_REQUIRE_EQ(do_access(&rules, &headers, "test@plop", &msg, &qualifier), ACT_MODERATE);
+	ATF_REQUIRE(qualifier == NULL);
+	ATF_REQUIRE_STREQ(msg, "access - A mail from \"test@plop\" with header \"from: toto@bla\" was moderated by rule #0 \"moderate- ^from: toto@\"");
+	tll_push_front(rules, "moderate-meh ^from: toto@");
+	ATF_REQUIRE_EQ(do_access(&rules, &headers, "test@plop", &msg, &qualifier), ACT_MODERATE);
+	ATF_REQUIRE_STREQ(qualifier, "meh");
+	ATF_REQUIRE_STREQ(msg, "access - A mail from \"test@plop\" with header \"from: toto@bla\" was moderated by rule #0 \"moderate-meh ^from: toto@\"");
+}
+
+ATF_TC_BODY(unistr_header_to_utf8, tc)
+{
+    const char *input = "=?UTF-8?B?SGVsbG8gd29ybGQ=?=";
+
+    char *out = unistr_header_to_utf8(input);
+
+    ATF_REQUIRE(out != NULL);
+    ATF_REQUIRE_STREQ(out, "Hello world");
+
+    out = unistr_header_to_utf8("=?UTF-8?B?Sm9obiBEb2U=?= <john.doe@example.com>");
+
+    ATF_REQUIRE(out != NULL);
+    ATF_REQUIRE_STREQ(out, "John Doe <john.doe@example.com>");
+
+    out = unistr_header_to_utf8("=?utf-8?b?5aSq6YOO?= <taro@example.com>");
+
+    ATF_REQUIRE(out != NULL);
+    ATF_REQUIRE_STREQ(out, "太郎 <taro@example.com>");
+
+    out = unistr_header_to_utf8("=?ISO-8859-1?Q?Jean-Pierre_Dupont?= <jean.pierre@example.com>");
+
+    ATF_REQUIRE(out != NULL);
+    ATF_REQUIRE_STREQ(out, "Jean-Pierre Dupont <jean.pierre@example.com>");
+
+    out = unistr_header_to_utf8("=?koi8-r?b?8NLJ18XUIM3J0g==?=");
+    ATF_REQUIRE(out != NULL);
+    ATF_REQUIRE_STREQ(out, "Привет мир");
+
+    out = unistr_header_to_utf8("=?us-ascii?Q?useless_but_why_not?=");
+    ATF_REQUIRE(out != NULL);
+    ATF_REQUIRE_STREQ(out, "useless but why not");
+}
+
 ATF_TP_ADD_TCS(tp)
 {
 	ATF_TP_ADD_TC(tp, random_int);
@@ -2948,6 +3092,10 @@ ATF_TP_ADD_TCS(tp)
 	ATF_TP_ADD_TC(tp, readlf);
 	ATF_TP_ADD_TC(tp, mod_get_addr_type);
 	ATF_TP_ADD_TC(tp, send_probe);
+	ATF_TP_ADD_TC(tp, parse_content_type);
+	ATF_TP_ADD_TC(tp, find_in_list);
+	ATF_TP_ADD_TC(tp, do_access);
+	ATF_TP_ADD_TC(tp, unistr_header_to_utf8);
 
 	return (atf_no_error());
 }
